@@ -26,7 +26,7 @@ import sys
 from decimal import Context, Decimal, localcontext
 from fractions import Fraction
 from functools import reduce
-from inspect import isclass, isfunction
+from inspect import isclass
 from uuid import UUID
 
 import attr
@@ -67,6 +67,7 @@ from hypothesis.internal.floats import (
 from hypothesis.internal.reflection import (
     define_function_signature,
     is_typed_named_tuple,
+    nicerepr,
     proxies,
     required_args,
 )
@@ -93,6 +94,7 @@ from hypothesis.searchstrategy.datetime import (
     TimedeltaStrategy,
 )
 from hypothesis.searchstrategy.deferred import DeferredStrategy
+from hypothesis.searchstrategy.functions import FunctionStrategy
 from hypothesis.searchstrategy.lazy import LazyStrategy
 from hypothesis.searchstrategy.misc import (
     BoolStrategy,
@@ -716,7 +718,7 @@ def lists(
     elements,  # type: SearchStrategy[Ex]
     min_size=0,  # type: int
     max_size=None,  # type: int
-    unique_by=None,  # type: Callable[..., Any]
+    unique_by=None,  # type: Union[Callable, Tuple[Callable, ...]]
     unique=False,  # type: bool
 ):
     # type: (...) -> SearchStrategy[List[Ex]]
@@ -729,10 +731,20 @@ def lists(
     object equality, as if unique_by was ``lambda x: x``. This comparison only
     works for hashable types.
 
-    if unique_by is not None it must be a function returning a hashable type
-    when given a value drawn from elements. The resulting list will satisfy the
-    condition that for ``i`` != ``j``, ``unique_by(result[i])`` !=
-    ``unique_by(result[j])``.
+    If unique_by is not None it must be a callable or tuple of callables
+    returning a hashable type when given a value drawn from elements. The
+    resulting list will satisfy the condition that for ``i`` != ``j``,
+    ``unique_by(result[i])`` != ``unique_by(result[j])``.
+
+    If ``unique_by`` is a tuple of callables the uniqueness will be respective
+    to each callable.
+
+    For example, the following will produce two columns of integers with both
+    columns being unique respectively.
+    .. code-block:: pycon
+
+        >>> twoints = st.tuples(st.integers(), st.integers())
+        >>> st.lists(twoints, unique_by=(lambda x: x[0], lambda x: x[1]))
 
     Examples from this strategy shrink by trying to remove elements from the
     list, and by shrinking each individual element of the list.
@@ -753,8 +765,19 @@ def lists(
     if max_size == 0:
         return builds(list)
     if unique_by is not None:
+        if not (callable(unique_by) or isinstance(unique_by, tuple)):
+            raise InvalidArgument(
+                "unique_by=%r is not a callable or tuple of callables" % (unique_by)
+            )
+        if callable(unique_by):
+            unique_by = (unique_by,)
+        if len(unique_by) == 0:
+            raise InvalidArgument("unique_by is empty")
+        for i, f in enumerate(unique_by):
+            if not callable(f):
+                raise InvalidArgument("unique_by[%i]=%r is not a callable" % (i, f))
         return UniqueListStrategy(
-            elements=elements, max_size=max_size, min_size=min_size, key=unique_by
+            elements=elements, max_size=max_size, min_size=min_size, keys=unique_by
         )
     return ListStrategy(elements, min_size=min_size, max_size=max_size)
 
@@ -1287,17 +1310,30 @@ def from_type(thing):
     # refactoring it's hard to do without creating circular imports.
     from hypothesis.searchstrategy import types
 
+    def as_strategy(strat_or_callable, thing):
+        # User-provided strategies need some validation, and callables even more
+        # of it.  We do this in three places, hence the helper function
+        if not isinstance(strat_or_callable, SearchStrategy):
+            assert callable(strat_or_callable)  # Validated in register_type_strategy
+            strategy = strat_or_callable(thing)
+        else:
+            strategy = strat_or_callable
+        if not isinstance(strategy, SearchStrategy):
+            raise ResolutionFailed(
+                "Error: %s was registered for %r, but returned non-strategy %r"
+                % (thing, nicerepr(strat_or_callable), strategy)
+            )
+        if strategy.is_empty:
+            raise ResolutionFailed("Error: %r resolved to an empty strategy" % (thing,))
+        return strategy
+
     if typing is not None:  # pragma: no branch
         if not isinstance(thing, type):
-            # At runtime, `typing.NewType` returns an identity function rather
-            # than an actual type, but we can check that for a possible match
-            # and then read the magic attribute to unwrap it.
-            if (
-                hasattr(thing, "__supertype__")
-                and hasattr(typing, "NewType")
-                and isfunction(thing)
-                and getattr(thing, "__module__", 0) == "typing"
-            ):
+            if types.is_a_new_type(thing):
+                # Check if we have an explicitly registered strategy for this thing,
+                # resolve it so, and otherwise resolve as for the base type.
+                if thing in types._global_type_lookup:
+                    return as_strategy(types._global_type_lookup[thing], thing)
                 return from_type(thing.__supertype__)
             # Under Python 3.6, Unions are not instances of `type` - but we
             # still want to resolve them!
@@ -1321,12 +1357,7 @@ def from_type(thing):
     # lookup may be a strategy or a function from type -> strategy; and we
     # convert empty results into an explicit error.
     if thing in types._global_type_lookup:
-        strategy = types._global_type_lookup[thing]
-        if not isinstance(strategy, SearchStrategy):
-            strategy = strategy(thing)  # type: ignore
-        if strategy.is_empty:  # type: ignore
-            raise ResolutionFailed("Error: %r resolved to an empty strategy" % (thing,))
-        return strategy  # type: ignore
+        return as_strategy(types._global_type_lookup[thing], thing)
     # If there's no explicitly registered strategy, maybe a subtype of thing
     # is registered - if so, we can resolve it to the subclass strategy.
     # We'll start by checking if thing is from from the typing module,
@@ -1340,19 +1371,13 @@ def from_type(thing):
     # type.  For example, `Number -> integers() | floats()`, but bools() is
     # not included because bool is a subclass of int as well as Number.
     strategies = [
-        v if isinstance(v, SearchStrategy) else v(thing)  # type: ignore
+        as_strategy(v, thing)
         for k, v in types._global_type_lookup.items()
         if isinstance(k, type)
         and issubclass(k, thing)
         and sum(types.try_issubclass(k, typ) for typ in types._global_type_lookup) == 1
     ]
-    empty = ", ".join(repr(s) for s in strategies if s.is_empty)
-    if empty:
-        raise ResolutionFailed(
-            "Could not resolve %s to a strategy; consider using "
-            "register_type_strategy" % empty
-        )
-    elif strategies:
+    if strategies:
         return one_of(strategies)
     # If we don't have a strategy registered for this type or any subtype, we
     # may be able to fall back on type annotations.
@@ -2027,6 +2052,13 @@ def runner(default=not_set):
 
 
 class DataObject(object):
+    """This type only exists so that you can write type hints for tests using
+    the :func:`~hypothesis.strategies.data` strategy.  Do not use it directly!
+    """
+
+    # Note that "only exists" here really means "is only exported to users",
+    # but we want to treat it as "semi-stable", not document it as "public API".
+
     def __init__(self, data):
         self.count = 0
         self.conjecture_data = data
@@ -2035,6 +2067,7 @@ class DataObject(object):
         return "data(...)"
 
     def draw(self, strategy, label=None):
+        # type: (SearchStrategy[Ex], Any) -> Ex
         result = self.conjecture_data.draw(strategy)
         self.count += 1
         if label is not None:
@@ -2071,17 +2104,14 @@ class DataStrategy(SearchStrategy):
 
     def __not_a_first_class_strategy(self, name):
         raise InvalidArgument(
-            (
-                "Cannot call %s on a DataStrategy. You should probably be "
-                "using @composite for whatever it is you're trying to do."
-            )
-            % (name,)
+            "Cannot call %s on a DataStrategy. You should probably be using "
+            "@composite for whatever it is you're trying to do." % (name,)
         )
 
 
 @cacheable
 def data():
-    # type: () -> SearchStrategy[Any]
+    # type: () -> SearchStrategy[DataObject]
     """This isn't really a normal strategy, but instead gives you an object
     which can be used to draw data interactively from other strategies.
 
@@ -2186,3 +2216,72 @@ def emails():
     return builds(u"{}@{}".format, local_part, domains()).filter(
         lambda addr: len(addr) <= 254
     )
+
+
+# Mypy can't yet handle default values with generic types or typevars, but the
+# @overload workaround from https://github.com/python/mypy/issues/3737 doesn't
+# work with @composite functions - Mypy can't see that the function implements
+# `(Any, Callable[..., T], SearchStrategy[T]) -> Callable[..., T]`
+
+
+@defines_strategy
+def functions(like=lambda: None, returns=none()):
+    """A strategy for functions, which can be used in callbacks.
+
+    The generated functions will mimic the interface of ``like``, which must
+    be a callable (including a class, method, or function).  The return value
+    for the function is drawn from the ``returns`` argument, which must be a
+    strategy.
+
+    Note that the generated functions do not validate their arguments, and
+    may return a different value if called again with the same arguments.
+
+    Generated functions can only be called within the scope of the ``@given``
+    which created them.  This strategy does not support ``.example()``.
+    """
+    check_type(SearchStrategy, returns)
+    if not callable(like):
+        raise InvalidArgument(
+            "The first argument to functions() must be a callable to imitate, "
+            "but got non-callable like=%r" % (nicerepr(like),)
+        )
+    return FunctionStrategy(like, returns)
+
+
+@composite
+def slices(draw, size):
+    """Generates slices that will select indices up to the supplied size
+
+    Generated slices will have start and stop indices that range from 0 to size - 1
+    and will step in the appropriate direction. Slices should only produce an empty selection
+    if the start and end are the same.
+
+    Examples from this strategy shrink toward 0 and smaller values
+    """
+    check_valid_integer(size)
+    if size is None or size < 1:
+        raise InvalidArgument("size=%r must be at least one" % size)
+
+    min_start = min_stop = 0
+    max_start = max_stop = size
+    min_step = 1
+    # For slices start is inclusive and stop is exclusive
+    start = draw(integers(min_start, max_start) | none())
+    stop = draw(integers(min_stop, max_stop) | none())
+
+    # Limit step size to be reasonable
+    if start is None and stop is None:
+        max_step = size
+    elif start is None:
+        max_step = stop
+    elif stop is None:
+        max_step = start
+    else:
+        max_step = abs(start - stop)
+
+    step = draw(integers(min_step, max_step or 1))
+
+    if (stop or 0) < (start or 0):
+        step *= -1
+
+    return slice(start, stop, step)
